Type: Package
Package: ReactomeGSA
Title: Client for the Reactome Analysis Service for comparative
        multi-omics gene set analysis
Version: 1.27.0
Authors@R: 
    person("Johannes", "Griss", , "johannes.griss@meduniwien.ac.at", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2206-9511"))
Author: Johannes Griss [aut, cre]
    (<https://orcid.org/0000-0003-2206-9511>)
Maintainer: Johannes Griss <johannes.griss@meduniwien.ac.at>
Description: The ReactomeGSA packages uses Reactome's online analysis
    service to perform a multi-omics gene set analysis. The main advantage
    of this package is, that the retrieved results can be visualized using
    REACTOME's powerful webapplication.  Since Reactome's analysis service
    also uses R to perfrom the actual gene set analysis you will get
    similar results when using the same packages (such as limma and edgeR)
    locally.  Therefore, if you only require a gene set analysis,
    different packages are more suited.
License: MIT + file LICENSE
URL: https://github.com/reactome/ReactomeGSA
BugReports: https://github.com/reactome/ReactomeGSA/issues
Imports: Biobase, BiocSingular, dplyr, ggplot2, gplots, httr, igraph,
        jsonlite, methods, progress, RColorBrewer,
        SummarizedExperiment, tidyr
Suggests: devtools, knitr, ReactomeGSA.data, rmarkdown, scater, scran,
        scRNAseq, scuttle, Seurat (>= 3.0), SingleCellExperiment,
        testthat
VignetteBuilder: knitr
biocViews: GeneSetEnrichment, Proteomics, Transcriptomics,
        SystemsBiology, GeneExpression, Reactome
Encoding: UTF-8
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/ReactomeGSA
git_branch: devel
git_last_commit: bf91539
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-11
NeedsCompilation: no
Packaged: 2026-05-11 23:45:51 UTC; biocbuild
