# Generated by roxygen2: do not edit by hand

export(ReactomeAnalysisRequest)
export(ReactomeAnalysisResult)
export(add_dataset)
export(analyse_sc_clusters)
export(find_public_datasets)
export(generate_metadata)
export(generate_pseudo_bulk_data)
export(get_public_species)
export(get_reactome_analysis_result)
export(get_reactome_analysis_status)
export(get_reactome_data_types)
export(get_reactome_methods)
export(get_result)
export(load_public_dataset)
export(open_reactome)
export(pathways)
export(perform_reactome_analysis)
export(plot_correlations)
export(plot_gsva_heatmap)
export(plot_gsva_pathway)
export(plot_gsva_pca)
export(plot_heatmap)
export(plot_volcano)
export(reactome_links)
export(remove_dataset)
export(result_types)
export(set_method)
export(set_parameters)
export(start_reactome_analysis)
exportClasses(ReactomeAnalysisResult)
exportMethods(generate_metadata)
exportMethods(generate_pseudo_bulk_data)
exportMethods(names)
exportMethods(print)
exportMethods(show)
import(httr)
import(methods)
importClassesFrom(Biobase,AnnotatedDataFrame)
importClassesFrom(Biobase,ExpressionSet)
importClassesFrom(Biobase,MIAME)
importFrom(dplyr,any_of)
importFrom(dplyr,select)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(gplots,heatmap.2)
importFrom(methods,is)
importFrom(tidyr,pivot_longer)
importFrom(utils,URLencode)
